# -*- coding: utf-8 -*-

import sys
import xml.dom.minidom
import xml.sax.saxutils

status_table = {
    'online':       'NLN',
    'away':         'AWY',
    'busy':         'BSY',
    'brb':          'BRB',
    'phone':        'PHN',
    'lunch':        'LUN',
    'invisible':    'HDN',
    'idle':         'IDL',
    'offline':      'FLN',
}

reverse_status = {
    'NLN':        'online',
    'AWY':        'away',
    'BSY':        'busy',
    'BRB':        'brb',
    'PHN':        'phone',
    'LUN':        'lunch',
    'HDN':        'invisible',
    'IDL':        'idle',
    'FLN':        'offline',
}

dic = {
    '\"'    :    '&quot;',
    '\''    :    '&apos;'
}

dic_inv = {
    '&quot;'    :'\"',
    '&apos;'    :'\''
}

debugFlag = False
# a list of sections that wont be shown on debug
disabled_sections = []
def debug(data, section='default'):
    '''print debug data to the console if the debug flag is set
    and section not in disabled_sections
    '''
    
    if debugFlag and section not in disabled_sections:
        if str( data ).endswith( '\n' ):
            sys.stdout.write( "\r" + str( data ).__repr__()[1:-1].replace( '\\r\\n', '\n' ).replace( '\\t', '\t' ) )
        else:
            sys.stdout.write( "\r" + str( data ).__repr__()[1:-1].replace( '\\r\\n', '\n' ).replace( '\\t', '\t' ) + '\n')
            
        sys.stdout.flush()

def escape(string):
    return xml.sax.saxutils.escape(string, dic)

def unescape(string):
    return xml.sax.saxutils.unescape(string, dic_inv)

def dumpXmlToFile(string, file_name):
    dom = xml.dom.minidom.parseString(string)
    f = file(file_name, 'w')
    f.write(dom.toprettyxml())
    f.close()

def parseSoapFault( response ):
    '''parse the xml response to extract the fault'''
    body = response.body
    
    ml = xml.dom.minidom.parseString( body )
    try:
        fault = ml.getElementsByTagName( 'soap:Fault' )[ 0 ]
    except IndexError:
        return ''

    try:
        return fault.getElementsByTagName( 'errorstring' )[ 0 ].childNodes[ 0 ].data
    except IndexError:
        return fault.getElementsByTagName( 'faultstring' )[ 0 ].childNodes[ 0 ].data

class LoginError( Exception ):
    def __init__( self, value ):
        self.value = value
        
    def __str__( self ):
        return "Couldn't perform login: " + repr( self.value )
    
class AuthError( Exception ):
    def __init__( self, value ):
        self.value = value
        
    def __str__( self ):
        return 'Authentication error: ' + repr( self.value )
    
class RedirectionError( Exception ):
    def __init__( self, value ):
        self.value = value
        
    def __str__( self ):
        return 'Redirection error: ' + repr( self.value )
    
class ProtocolError( Exception ):
    def __init__( self, value ):
        self.value = value
        
    def __str__( self ):
        return 'Protocol error: ' + repr( self.value )
    
class SoapError( Exception ):
    def __init__( self, value, response ):
        self.value = value
        
        self.responseStatus = response.status
        body = response.body
        
        ml = xml.dom.minidom.parseString( body )
        fault = ml.getElementsByTagName( 'soap:Fault' )[ 0 ]
        try:
            self.faultcode = fault.getElementsByTagName( 'errorstring' )[ 0 ].childNodes[ 0 ].data
        except IndexError:
            self.faultcode = fault.getElementsByTagName( 'faultstring' )[ 0 ].childNodes[ 0 ].data
        
    def __str__( self ):
        return repr( self.value ) + ' ' + repr( self.responseStatus ) \
                    + ': ' + repr( self.faultcode )


