import gtk
import gobject
import gettext

_ = gettext.gettext

class FilterPanel( gtk.Notebook ):
    '''a tabbed widget to manage the filters'''



    def __init__( self, signalDict, emailList ):
        gtk.Notebook.__init__( self )

        self.eventFilters = EventFilterTab( signalDict )
        self.eventFilters.show()
        self.append_page( self.eventFilters, gtk.Label( _( 'Event filters' ) ) )

        self.mailFilters = None

        self.clearMails( emailList )

    def clearMails( self, emailList ):
        if self.mailFilters:
            self.remove_page(-1)
    
        self.mailFilters = MailFilterTab( emailList )
        self.mailFilters.show()
        self.append_page( self.mailFilters, gtk.Label( _( 'Mail filters' ) ) )
        

class EventFilterTab( gtk.ScrolledWindow ):
    '''a widget that hold the checkbox to activate/deactivate
    the display of each signal'''
    
    __gsignals__ = {
        # the parameters are the event name and the value (True, False)
        'event-filter-changed' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_STRING, gobject.TYPE_BOOLEAN))
    }

    def __init__( self, signalDict ):
        '''signal dict is one like the eventNames on LogView'''
        gtk.ScrolledWindow.__init__( self )
        self.set_policy( gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC )

        self.vbox = gtk.VBox()
        self.vbox.set_border_width( 5 )
        self.vbox.set_spacing( 2 )

        self.checks = []

        self.bSelectAll = gtk.Button( _( 'Select all' ) )
        self.bSelectAll.connect( 'clicked', self.onSelectAllClicked )
        
        self.bDeselectAll = gtk.Button( _( 'Deselect all' ) )
        self.bDeselectAll.connect( 'clicked', self.onDeselectAllClicked )

        self.vbox.pack_end( self.bSelectAll, False, False )
        self.vbox.pack_end( self.bDeselectAll, False, False )

        for (eventName, niceName) in signalDict.iteritems():
            check = gtk.CheckButton( niceName, False )
            check.connect( 'toggled', self.onCheckToggled, eventName )
            self.checks.append( check )
            self.vbox.pack_start( check, False, False )

        self.add_with_viewport( self.vbox )
        self.vbox.show_all()

    def onCheckToggled( self, widget, signal ):
        self.emit( 'event-filter-changed', signal, widget.get_active() )

    def onSelectAllClicked( self, widget ):
        for check in self.checks:
            check.set_active( True )

    def onDeselectAllClicked( self, widget ):
        for check in self.checks:
            check.set_active( False )

class MailFilterTab( gtk.ScrolledWindow ):
    '''a widget that let te user filter the emails'''

    __gsignals__ = {
        # the parameter is the mail, and if it was selected or deselected
        'mail-filter-changed' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_STRING, gobject.TYPE_BOOLEAN)) 
    }

    def __init__( self, emailList ):
        gtk.ScrolledWindow.__init__( self )
        self.set_policy( gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC )

        self.vbox = gtk.VBox()
        self.vbox.set_border_width( 5 )
        self.vbox.set_spacing( 2 )

        self.checks = []

        self.bSelectAll = gtk.Button( _( 'Select all' ) )
        self.bSelectAll.connect( 'clicked', self.onSelectAllClicked )
        
        self.bDeselectAll = gtk.Button( _( 'Deselect all' ) )
        self.bDeselectAll.connect( 'clicked', self.onDeselectAllClicked )

        self.vbox.pack_end( self.bSelectAll, False, False )
        self.vbox.pack_end( self.bDeselectAll, False, False )

        for mail in emailList:
            self.addMail( mail )

        self.add_with_viewport( self.vbox )

        self.vbox.show_all()

    def addMail( self, mail ):
        '''add a checkbox with the given email'''
        check = gtk.CheckButton( mail, False )
        check.connect( 'toggled', self.onCheckToggled, mail )
        self.checks.append( check )
        self.vbox.pack_start( check, False, False )
        self.vbox.show_all()

    def onCheckToggled( self, widget, signal ):
        self.emit( 'mail-filter-changed', signal, widget.get_active() )

    def onSelectAllClicked( self, widget ):
        for check in self.checks:
            check.set_active( True )

    def onDeselectAllClicked( self, widget ):
        for check in self.checks:
            check.set_active( False )

gobject.type_register( FilterPanel )
gobject.type_register( EventFilterTab )
gobject.type_register( MailFilterTab )
