import gtk
import gettext

_ = gettext.gettext

class LogModel( gtk.ListStore ):
    '''a model to show the log data'''

    def __init__( self ):
        # datetime, event, email, extra, LogLine, visible
        gtk.ListStore.__init__( self, str, str, str, str, object, bool )


class LogView( gtk.TreeView ):
    '''the widget that hold the log data'''

    eventNames = {}
    eventNames[ 'self-nick-changed' ] = _( 'Our nick changed' )
    eventNames[ 'status-change' ] = _( 'Status change' )
    eventNames[ 'personal-message-changed' ] = _( 'Personal message changed' )
    eventNames[ 'self-status-changed' ] = _( 'Our status changed' )
    eventNames[ 'self-personal-message-changed' ] = _( 'Our personal message changed' )
    eventNames[ 'self-current-media-changed' ] = _( 'Our current media changed' )
    eventNames[ 'new-mail-notification' ] = _( 'New mail received' )
    eventNames[ 'nick-changed' ] = _( 'Nick changed' )
    eventNames[ 'contact-status-change' ] = _( 'Status change' )
    eventNames[ 'user-online' ] = _( 'User online' )
    eventNames[ 'user-offline' ] = _( 'User offline' )

    def __init__( self ):
        gtk.TreeView.__init__( self, LogModel() )

        count = 0

        self.visibleEvents = []
        self.visibleMails = []

        for i in [ _( 'Date' ), _( 'Event' ), _( 'Mail' ), _( 'Extra' ) ]:
            column = gtk.TreeViewColumn()
            column.set_title( i )
            
            cellRenderer = gtk.CellRendererText()
            column.pack_start( cellRenderer )
            column.add_attribute( cellRenderer, 'text', count )
           
            self.append_column( column )
            count += 1

        self.set_headers_visible( True )
        self.set_reorderable( True )
       
        self.model = self.get_model()
        self.modelFilter = self.model.filter_new()
        self.modelFilter.set_visible_func( self.filter )

        self.set_model( self.modelFilter )

    def add( self, logLine ):
        '''add a Log.LogLine to the model'''

        if logLine.event in LogView.eventNames:
            event = LogView.eventNames[ logLine.event ]
        else:
            event = logLine.event

        self.model.append( [ str( logLine.date ), event, logLine.email, logLine.formatExtra(), logLine, True ] )
    
    def filter( self, model, iter ):
        row = model[ iter ]
        logLine = row[ 4 ] 
        if logLine and logLine.event in self.visibleEvents and logLine.email in self.visibleMails:
            return True

        return False

    def refilter( self ):
        self.modelFilter.refilter()

    def addVisibleMail( self, mail ):
        if mail not in self.visibleMails:
            self.visibleMails.append( mail )
            self.refilter()

    def delVisibleMail( self, mail ):
        if mail in self.visibleMails:
            self.visibleMails.remove( mail  )
            self.refilter()
    
    def addVisibleEvent( self, event ):
        if event not in self.visibleEvents:
            self.visibleEvents.append( event )
            self.refilter()

    def delVisibleEvent( self, event ):
        if event in self.visibleEvents:
            self.visibleEvents.remove( event  )
            self.refilter()

