# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
import os
import gobject

import Plugin
import currentSong

class MainClass( Plugin.Plugin ):
    '''Main plugin class'''
    
    def __init__( self, controller, msn ):
        '''Contructor'''
        
        Plugin.Plugin.__init__( self, controller, msn, 1000 )
        self.theme = controller.theme
        
        self.description = _( 'Show a personal message with the current song played in multiple players.' )
        self.authors = { 'Mariano Guerra' : 'luismarianoguerra at gmail dot com' }
        self.website = 'http://emesene.org'
        self.displayName = _( 'CurrentSong' )
        self.name = 'CurrentSong'
        
        self.config = controller.getConfig()
        self.config.readPluginConfig(self.name)
        self.usingDummy = True
        self.setPlayer( self.config.getPluginValue( self.name, 'player', 'CurrentSong' ) )
        self.player.setStyle( self.config.getPluginValue( self.name, 'style', '%title - %artist (%album)' ) )
        self.avatar = bool(self.config.getPluginValue(self.name, 'avatar', "False"))
        
        self.updateCustomConf()
                
        self.timeout = 0
    
    def start( self ):
        '''start the plugin'''
        
        if not self.usingDummy:
            self.player.start()
        self.timeout = gobject.timeout_add( self.period, self.action )
        self.enabled = True
        
    def stop( self ):    
        '''stop the plugin'''
        
        if not self.usingDummy:
            self.player.stop()
        if self.timeout:
            gobject.source_remove(self.timeout)
        self.msn.changeCurrentMedia( "" )
        self.enabled = False
        
    def action( self ):
        if not self.enabled: return True
        
        if not self.usingDummy:
            try:
                check = self.player.check()
            except Exception, e:
                print "Error in player.check():", e
                self.msn.changeCurrentMedia( "" )
                return True
            if check and self.controller.getMediaEnabled(): 
                self.msn.changeCurrentMedia( self.player.getCurrentSong() )
                if self.avatar:
                    path = self.player.getCoverPath()
                    if path != None and os.path.exists(path):
                        self.controller.changeAvatar(path)
        else:
            self.setPlayer( self.config.getPluginValue( self.name, 'player', 'CurrentSong' ) )

        return True
        
    def is_on_path(self, fname):
        for p in os.environ['PATH'].split(os.pathsep):
            if os.path.isfile(os.path.join(p, fname)):
                return True
        
    def getPluginNames( self ):
        return [ x for x in dir( currentSong ) \
                 if not x.startswith( "__" ) \
                 and x != 'CurrentSong' ]
        
    def setPlayer( self, name ):
        '''try to set the player object to the name given by the name parameter
        if not set the dummy player to have something to play with'''
        
        players = self.getPluginNames()
        self.playername = name

        error = '' 
        if name in players:
            try:
                # ugly but i dont have to make a dict with the classes :D
                self.player = eval( "currentSong." + name + "()" )
                self.usingDummy = False
                self.action()
            except Exception, e:
                print "error initializing CurrentSong player:", e
                error = e
            else:
                return
        self.player = currentSong.CurrentSong()
        self.usingDummy = True
        if error:
            return error
        
    def check( self ):
        '''check if everything is OK to start the plugin
        return a tuple whith a boolean and a message
        if OK -> ( True , 'some message' )
        else -> ( False , 'error message' )'''
        
        return self.player.getStatus()
   
    def updateCustomConf( self ):
        for key in self.player.customConfig.keys():
            self.player.customConfig[key] = self.config.getPluginValue( self.name, str(self.playername + "_" + key), self.player.customConfig[key] )     
        self.player.updateConfig()

    def configure( self ):
        '''display a configuration dialog'''

        style = self.config.getPluginValue(self.name, 'style', '%title - %artist (%album)')
        player = self.config.getPluginValue(self.name, 'player', 'Amarok')
        avatar = ((self.config.getPluginValue(self.name, 'avatar', '0')) == '1' )
        #self.updateCustomConf()
        l = []
        # name, optionType, label, description, value, options
        l.append(Plugin.Option('style', str, 'Display style', 'Change display style of the song you are listening, possible variables are %title, %artist and %album', style))
        l.append(Plugin.Option('player', list, 'Audio player', 'select the audio player that you are using', player, self.getPluginNames(), self.playerChanged))
        l.append(Plugin.Option('avatar', bool, 'Change Avatar', 'Change Avatar', avatar))
        custom = Plugin.Option('custom', dict, 'Custom config', 'Custom config', None, self.player.customConfig)
        l.append(custom)

        configWindow = Plugin.ConfigWindow( _( 'Config Plugin' ), l )
        self.customConfigWidget = custom.widget
        response = configWindow.run()
        if response != None:
            if response.has_key( 'player' ):
                self.setPlayer( response[ 'player' ].value )
                self.config.setPluginValue( self.name, 'player', response[ 'player' ].value )
                
            if response.has_key( 'style' ):
                self.player.setStyle( response[ 'style' ].value )
                self.config.setPluginValue( self.name, 'style', response[ 'style' ].value )
            if response.has_key('avatar'):
                self.avatar = response['avatar'].value
                self.config.setPluginValue(self.name, 'avatar', str(int(response['avatar'].value)))
            if response.has_key('custom'):
                self.player.customConfig = self.customConfigWidget.data
                for key in self.player.customConfig.keys():
                    self.config.setPluginValue(self.name, str(self.playername + "_" + key), self.player.customConfig[key])
                self.player.updateConfig()    
        return True

    def playerChanged( self, combo ):
        newplayer = combo.get_active_text()
        self.setPlayer(newplayer)
        self.updateCustomConf() 
        self.customConfigWidget.refresh(self.player.customConfig)

